# Scola version 1.1.0
# ScolaHistories.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of ScolaWizard.
# ScolaWizard is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# ScolaWizard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with ScolaWizard.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Commands
# 3) Histories
# 4) History managers

import sys
import os

# 2) Commands
# -----------

class ScolaCommand:
  NEW_PROCESS = "NEW_PROCESS"
  KILL_PROCESS = "KILL_PROCESS"
  NEXT = "NEXT"
  CHOICE = "CHOICE"

  def __init__(self, type):
    self._type = type
    self._arguments = []
    self._comment = None

  @staticmethod
  def NewNewProcess(locationName):
    self = ScolaCommand(ScolaCommand.NEW_PROCESS)
    self.AppendArgument(locationName)
    return self

  @staticmethod
  def NewKillProcess(processNumber):
    self = ScolaCommand(ScolaCommand.KILL_PROCESS)
    self.AppendArgument(processNumber)
    return self

  @staticmethod
  def NewNext(processNumber):
    self = ScolaCommand(ScolaCommand.NEXT)
    self.AppendArgument(processNumber)
    return self

  @staticmethod
  def NewChoice(processNumber, branchName):
    self = ScolaCommand(ScolaCommand.CHOICE)
    self.AppendArgument(processNumber)
    self.AppendArgument(branchName)
    return self

  def GetType(self):
    return self._type

  def GetNumberOfArguments(self):
    return len(self._arguments)

  def GetArgumentAsLocationName(self, index):
    if index<0 or index>=self.GetNumberOfArguments():
      return None
    return self._arguments[index]

  def GetArgumentAsProcessNumber(self, index):
    if index<0 or index>=self.GetNumberOfArguments():
      return None
    return self._arguments[index]

  def AppendArgument(self, argument):
    self._arguments.append(argument)

  def GetComment(self):
    return self._comment
  def SetComment(self, comment):
    self._comment = comment

# 3) History
# ----------

class ScolaHistory:
  def __init__(self):
    self._commands = []

  def AppendCommand(self, command):
    self._commands.append(command)

  def GetSize(self):
    return len(self._commands)

  def GetCommand(self, index):
    if index<0 or index>=self.GetSize():
      return None
    return self._commands[index]

# 4) History managers
# -------------------

class ScolaHistoryManager:
  def __init__(self, number, location):
    self._histories = []

  def AppendHistory(self, history):
    self._histories.append(history)

  def GetNumberOfHistories(self):
    return len(self._histories)

  def GetHistory(self, index):
    if index<0 or index>=self.GetNumberOfHistories():
      return None
    return self._histories[index]
