# Scola version 1.1.0
# ScolaConstants.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of ScolaWizard.
# ScolaWizard is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# ScolaWizard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with ScolaWizard.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Constants

# 1) Imported modules
# -------------------

import string
import sys
import math

# 2) Constants
# ------------

class ScolaConstant():
  NONE = "NONE"
  TRUE = "TRUE"
  FALSE = "FALSE"
  INTEGER = "INTEGER"
  REAL = "REAL"
  SYMBOL = "SYMBOL"
  STRING = "STRING"

  _None = None
  _True = None
  _False = None

  def __init__(self, type, value):
    self._type = type
    self._value = value

  @staticmethod
  def NewNone():
    if ScolaConstant._None==None:
      ScolaConstant._None = ScolaConstant(ScolaConstant.NONE, None)
    return ScolaConstant._None

  @staticmethod
  def NewTrue():
    if ScolaConstant._True==None:
      ScolaConstant._True = ScolaConstant(ScolaConstant.TRUE, True)
    return ScolaConstant._True
  @staticmethod
  def NewFalse():
    if ScolaConstant._False==None:
      ScolaConstant._False = ScolaConstant(ScolaConstant.FALSE, False)
    return ScolaConstant._False
  @staticmethod
  def NewBoolean(value):
    if value==None:
      return ScolaConstant.NewNone()
    if value:
      return ScolaConstant.NewTrue()
    return ScolaConstant.NewFalse()

  @staticmethod
  def NewInteger(value):
    if value==None:
      return ScolaConstant.NewNone()
    return ScolaConstant(ScolaConstant.INTEGER, value)

  @staticmethod
  def NewReal(value):
    if value==None:
      return ScolaConstant.NewNone()
    return ScolaConstant(ScolaConstant.REAL, value)

  @staticmethod
  def NewSymbol(value):
    if value==None:
      return ScolaConstant.NewNone()
    return ScolaConstant(ScolaConstant.SYMBOL, value)

  @staticmethod
  def NewString(value):
    if value==None:
      return ScolaConstant.NewNone()
    return ScolaConstant(ScolaConstant.STRING, value)

  @staticmethod
  def Copy(constant):
    if constant.IsBoolean():
      return ScolaConstant.NewBoolean(constant._value)
    if constant.IsInteger():
      return ScolaConstant.NewInteger(constant._value)
    if constant.IsReal():
      return ScolaConstant.NewReal(constant._value)
    if constant.IsSymbol():
      return ScolaConstant.NewSymbol(constant._value)
    if constant.IsString():
      return ScolaConstant.NewString(constant._value)
    return ScolaConstant.NewNone()

  def GetType(self):
    return self._type
  def IsNone(self):
    return self._type==ScolaConstant.NONE
  def IsFalse(self):
    return self._type==ScolaConstant.FALSE
  def IsTrue(self):
    return self._type==ScolaConstant.TRUE
  def IsBoolean(self):
    return self.IsFalse() or self.IsTrue()
  def IsInteger(self):
    return self._type==ScolaConstant.INTEGER
  def IsReal(self):
    return self._type==ScolaConstant.REAL
  def IsNumber(self):
    return self.IsInteger() or self.IsReal()
  def IsSymbol(self):
    return self._type==ScolaConstant.SYMBOL
  def IsString(self):
    return self._type==ScolaConstant.STRING

  def GetValueAsBoolean(self):
    if self._type!=ScolaConstant.FALSE and self._type!=ScolaConstant.TRUE:
      return None
    return self._value
  def GetValueAsInteger(self):
    if self._type!=ScolaConstant.INTEGER:
      return None
    return self._value
  def GetValueAsReal(self):
    if self._type!=ScolaConstant.INTEGER and self._type!=ScolaConstant.REAL:
      return None
    return self._value
  def GetValueAsSymbol(self):
    if self._type!=ScolaConstant.SYMBOL:
      return None
    return self._value
  def GetValueAsString(self):
    if self._type!=ScolaConstant.STRING:
      return None
    return self._value

  @staticmethod
  def AreEq(constant1, constant2):
    if constant1.IsNone() and constant2.IsNone():
      return True
    if constant1.IsBoolean() and constant2.IsBoolean():
      return constant1._value==constant2._value
    if constant1.IsNumber() and constant2.IsNumber():
      return constant1._value==constant2._value
    if constant1.IsSymbol() and constant2.IsSymbol():
      return constant1._value==constant2._value
    if constant1.IsString() and constant2.IsString():
      return constant1._value==constant2._value
    return False
  @staticmethod
  def AreDf(constant1, constant2):
    if constant1.IsNone() and constant2.IsNone():
      return False
    if constant1.IsBoolean() and constant2.IsBoolean():
      return constant1._value!=constant2._value
    if constant1.IsNumber() and constant2.IsNumber():
      return constant1._value!=constant2._value
    if constant1.IsSymbol() and constant2.IsSymbol():
      return constant1._value!=constant2._value
    if constant1.IsString() and constant2.IsString():
      return constant1._value!=constant2._value
    return True

  @staticmethod
  def And(constant1, constant2):
    if not constant1.IsBoolean() or not constant2.IsBoolean():
      return ScolaConstant.NewNone()
    return ScolaConstant.NewBoolean(constant1._value and constant2._value)
  @staticmethod
  def Or(constant1, constant2):
    if not constant1.IsBoolean() or not constant2.IsBoolean():
      return ScolaConstant.NewNone()
    return ScolaConstant.NewBoolean(constant1._value or constant2._value)
  @staticmethod
  def Not(constant):
    if not constant.IsBoolean():
      return ScolaConstant.NewNone()
    return ScolaConstant.NewBoolean(not constant._value)

  @staticmethod
  def Eq(constant1, constant2):
    if constant1.IsBoolean() and constant2.IsBoolean():
      return ScolaConstant.NewBoolean(constant1._value==constant2._value)
    if constant1.IsNumber() and constant2.IsNumber():
      return ScolaConstant.NewBoolean(constant1._value==constant2._value)
    if constant1.IsSymbol() and constant2.IsSymbol():
      return ScolaConstant.NewBoolean(constant1._value==constant2._value)
    if constant1.IsString() and constant2.IsString():
      return ScolaConstant.NewBoolean(constant1._value==constant2._value)
    return ScolaConstant.NewNone()
  @staticmethod
  def Df(constant1, constant2):
    if constant1.IsBoolean() and constant2.IsBoolean():
      return ScolaConstant.NewBoolean(constant1._value!=constant2._value)
    if constant1.IsNumber() and constant2.IsNumber():
      return ScolaConstant.NewBoolean(constant1._value!=constant2._value)
    if constant1.IsSymbol() and constant2.IsSymbol():
      return ScolaConstant.NewBoolean(constant1._value!=constant2._value)
    if constant1.IsString() and constant2.IsString():
      return ScolaConstant.NewBoolean(constant1._value!=constant2._value)
    return ScolaConstant.NewNone()
  @staticmethod
  def Lt(constant1, constant2):
    if constant1.IsNumber() and constant2.IsNumber():
      return ScolaConstant.NewBoolean(constant1._value<constant2._value)
    if constant1.IsString() and constant2.IsString():
      return ScolaConstant.NewBoolean(constant1._value<constant2._value)
    return ScolaConstant.NewNone()
  @staticmethod
  def Gt(constant1, constant2):
    if constant1.IsNumber() and constant2.IsNumber():
      return ScolaConstant.NewBoolean(constant1._value>constant2._value)
    if constant1.IsString() and constant2.IsString():
      return ScolaConstant.NewBoolean(constant1._value>constant2._value)
    return ScolaConstant.NewNone()
  @staticmethod
  def Leq(constant1, constant2):
    if constant1.IsNumber() and constant2.IsNumber():
      return ScolaConstant.NewBoolean(constant1._value<=constant2._value)
    if constant1.IsString() and constant2.IsString():
      return ScolaConstant.NewBoolean(constant1._value<=constant2._value)
    return ScolaConstant.NewNone()
  @staticmethod
  def Geq(constant1, constant2):
    if constant1.IsNumber() and constant2.IsNumber():
      return ScolaConstant.NewBoolean(constant1._value>=constant2._value)
    if constant1.IsString() and constant2.IsString():
      return ScolaConstant.NewBoolean(constant1._value>=constant2._value)
    return ScolaConstant.NewNone()

  @staticmethod
  def Opp(constant):
    if constant.IsInteger():
      return ScolaConstant.NewInteger(-constant._value)
    if constant.IsReal():
      return ScolaConstant.NewReal(-constant._value)
    return ScolaConstant.NewNone()
  @staticmethod
  def Inv(constant):
    if constant.IsNumber():
      return ScolaConstant.NewReal(1.0/constant._value)
    return ScolaConstant.NewNone()
  @staticmethod
  def Add(constant1, constant2):
    if constant1.IsInteger() and constant2.IsInteger():
      return ScolaConstant.NewInteger(constant1._value+constant2._value)
    if constant1.IsNumber() and constant2.IsNumber():
      return ScolaConstant.NewReal(constant1._value+constant2._value)
    return ScolaConstant.NewNone()
  @staticmethod
  def Sub(constant1, constant2):
    if constant1.IsInteger() and constant2.IsInteger():
      return ScolaConstant.NewInteger(constant1._value-constant2._value)
    if constant1.IsNumber() and constant2.IsNumber():
      return ScolaConstant.NewReal(constant1._value-constant2._value)
    return ScolaConstant.NewNone()
  @staticmethod
  def Mul(constant1, constant2):
    if constant1.IsInteger() and constant2.IsInteger():
      return ScolaConstant.NewInteger(constant1._value*constant2._value)
    if constant1.IsNumber() and constant2.IsNumber():
      return ScolaConstant.NewReal(constant1._value*constant2._value)
    return ScolaConstant.NewNone()
  @staticmethod
  def Div(constant1, constant2):
    if constant1.IsInteger() and constant2.IsInteger():
      return ScolaConstant.NewInteger(constant1._value//constant2._value)
    if constant1.IsNumber() and constant2.IsNumber():
      return ScolaConstant.NewReal(constant1._value/constant2._value)
    return ScolaConstant.NewNone()
  @staticmethod
  def Min(constant1, constant2):
    if constant1.IsNumber() and constant2.IsNumber():
      if constant1._value<=constant2._value:
        return ScolaConstant.Copy(constant1)
      return ScolaConstant.Copy(constant2)
    return ScolaConstant.NewNone()
  @staticmethod
  def Max(constant1, constant2):
    if constant1.IsNumber() and constant2.IsNumber():
      if constant1._value>=constant2._value:
        return ScolaConstant.Copy(constant1)
      return ScolaConstant.Copy(constant2)
    return ScolaConstant.NewNone()
  @staticmethod
  def Abs(constant):
    if constant.IsInteger():
      return ScolaConstant.NewInteger(abs(constant._value))
    if constant.IsReal():
      return ScolaConstant.NewReal(abs(constant._value))
    return ScolaConstant.NewNone()
  @staticmethod
  def Pow(constant1, constant2):
    if constant1.IsInteger() and constant2.IsInteger() and constant2._value>=0:
      return ScolaConstant.NewInteger(pow(constant1._value, constant2._value))
    if constant1.IsNumber() and constant2.IsNumber():
      return ScolaConstant.NewReal(pow(constant1._value, constant2._value))
    return ScolaConstant.NewNone()
  @staticmethod
  def Exp(constant):
    if constant.IsNumber():
      return ScolaConstant.NewReal(math.exp(constant._value))
    return ScolaConstant.NewNone()
  @staticmethod
  def Log(constant):
    if constant.IsNumber():
      return ScolaConstant.NewReal(math.log(constant._value))
    return ScolaConstant.NewNone()
  @staticmethod
  def Sqrt(constant):
    if constant.IsNumber():
      return ScolaConstant.NewReal(math.sqrt(constant._value))
    return ScolaConstant.NewNone()
  @staticmethod
  def Mod(constant1, constant2):
    if constant1.IsInteger() and constant2.IsInteger():
      return ScolaConstant.NewInteger(constant1._value % constant2._value)
    if constant1.IsNumber() and constant2.IsNumber():
      return ScolaConstant.NewReal(math.fmod(constant1._value, constant2._value))
    return ScolaConstant.NewNone()
  @staticmethod
  def Ceil(constant):
    if constant.IsNumber():
      return ScolaConstant.NewReal(math.ceil(constant._value))
    return ScolaConstant.NewNone()
  @staticmethod
  def Floor(constant):
    if constant.IsNumber():
      return ScolaConstant.NewReal(math.floor(constant._value))
    return ScolaConstant.NewNone()

  @staticmethod
  def Append(constant1, constant2):
    if constant1.IsString() and constant2.IsString():
      return ScolaConstant.NewString(constant1._value+constant2._value)
    return ScolaConstant.NewNone()

  @staticmethod
  def ToInteger(constant):
    if constant.IsFalse():
      return ScolaConstant.NewInteger(0)
    if constant.IsTrue():
      return ScolaConstant.NewInteger(1)
    if constant.IsInteger():
      return ScolaConstant.NewInteger(constant.GetValueAsInteger())
    if constant.IsReal():
      return ScolaConstant.NewInteger(int(constant.GetValueAsReal()))
    if constant.IsString():
      try:
        value = int(constant.GetValueAsString())
        return ScolaConstant.NewInteger(value)
      except:
        return ScolaConstant.NewNone()
    return ScolaConstant.NewNone()

  @staticmethod
  def ToReal(constant):
    if constant.IsFalse():
      return ScolaConstant.NewReal(0)
    if constant.IsTrue():
      return ScolaConstant.NewReal(1)
    if constant.IsInteger():
      return ScolaConstant.NewReal(constant.GetValueAsInteger())
    if constant.IsReal():
      return ScolaConstant.NewReal(constant.GetValueAsReal())
    if constant.IsString():
      try:
        value = float(constant.GetValueAsString())
        return ScolaConstant.NewReal(value)
      except:
        return ScolaConstant.NewNone()
    return ScolaConstant.NewNone()

  @staticmethod
  def ToString(constant):
    if constant.IsNone():
      return ScolaConstant.NewString("none")
    if constant.IsFalse():
      return ScolaConstant.NewString("false")
    if constant.IsTrue():
      return ScolaConstant.NewString("true")
    if constant.IsInteger():
      return ScolaConstant.NewString(str(constant.GetValueAsInteger()))
    if constant.IsReal():
      return ScolaConstant.NewString(str(constant.GetValueAsReal()))
    if constant.IsSymbol():
      return ScolaConstant.NewString(constant.GetValueAsSymbol())
    if constant.IsString():
      return ScolaConstant.NewString(constant.GetValueAsString())
    return ScolaConstant.NewNone()

