# Scola version 1.1.0
# ScolaText.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of ScolaWizard.
# ScolaWizard is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# ScolaWizard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with ScolaWizard.  If not, see <http://www.gnu.org/licenses/>.
 
# Table of Contents
# -----------------
# 1) Imported Modules
# 2) Scola Text

# 1) Imported Modules
# -------------------

import string
import sys
import re
import enum
import math
import random
import tkinter 
import tkinter.ttk
import tkinter.scrolledtext
import tkinter.filedialog
import tkinter.messagebox
import os.path

from ScolaLexer import *
from tkinter import font

# 2) Creator
# ----------

class ScolaText(tkinter.scrolledtext.Text):
  def __init__(self, *args, **kwargs):
    tkinter.Text.__init__(self, *args, **kwargs)
    monospaceFont = tkinter.font.Font(font='TkFixedFont')
    self.configure(font=monospaceFont)
    keywordFont = monospaceFont.copy()
    keywordFont.configure(weight="bold")
    self._keywords = ScolaLexer._Keywords.keys()
    self._keywordPattern = "(^|[ \t.]|\()("
    first = True
    for keyword in self._keywords:
      if first:
        first = False
      else:
        self._keywordPattern += "|"
      self._keywordPattern += keyword
    self._keywordPattern += ")($|[ \t.]|\))"
    self.tag_config("keyword", foreground="blue", font=keywordFont)
    self._stringPattern = '"[^"]*"'
    self.tag_config("string", foreground="purple")
    self._scriptCommentPattern = '#.*\n'
    commentFont = monospaceFont.copy()
    commentFont.configure(slant="italic")
    self.tag_config("comment", foreground="green", font=commentFont)
    self.tag_config("error", foreground="red")

  def HighlightCode(self):
    self.HighlightPattern(self._keywordPattern, "keyword")
    self.HighlightPattern(self._stringPattern, "string")

  def HighlightScript(self):
    self.HighlightPattern(self._scriptCommentPattern, "comment")

  def HighlightErrorMessage(self, start="1.0", end="end"):
    self.HighlightPattern(".*", "error", start, end)

  def HighlightPattern(self, pattern, tag, start="1.0", end="end"):
    start = self.index(start)
    end = self.index(end)
    self.mark_set("matchStart", start)
    self.mark_set("matchEnd", start)
    self.mark_set("searchLimit", end)
    count = tkinter.IntVar()
    while True:
      index = self.search(pattern, "matchEnd", "searchLimit", count=count, regexp=True) # the pattern is always considered as a regexp
      if index=="":
        break
      if count.get()==0:
        break # degenerate pattern which matches zero-length strings
      self.mark_set("matchStart", index)
      self.mark_set("matchEnd", "%s+%sc" % (index, count.get()))
      self.tag_add(tag, "matchStart", "matchEnd")